package com.loit.common.script;

import com.loit.common.script.dto.*;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.common.utils.excel.ImportExcel;
import com.loit.common.utils.file.FileUtils;
import com.loit.common.utils.freemarker.FreeMarkerUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.http.entity.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.*;


/**
 * logstash-es-sone.conf
 *
 * logstash-es-portal.conf
 */
@Slf4j
public class GeneratorLogstashScript {


    protected static String root_path = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\bin";

    protected static String root_path_full = root_path + "\\elk";

    protected static String time_loit_servers = "portal,getway,notice,file,version,gray-server,mobile-portal";

    public static void main(String[] args) {

        try {
            String filePathStr = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\deployInfo.xlsx";

            File pdfFile = new File(filePathStr);
            FileInputStream fileInputStream = new FileInputStream(pdfFile);
            MultipartFile multipartFile = new MockMultipartFile(pdfFile.getName(), pdfFile.getName(),
                    ContentType.APPLICATION_OCTET_STREAM.toString(), fileInputStream);

            ImportExcel ei = new ImportExcel(multipartFile, 1, 0);
            List<DeployInfoDataDTO> grayVersionDataDTOList = ei.getDataList(DeployInfoDataDTO.class);

            if (ListUtil.isEmpty(grayVersionDataDTOList)) {
                return;
            }

            List<DeployInfoDataDTO> portalServerList = new ArrayList<>();
            List<DeployInfoDataDTO> businessServerList = new ArrayList<>();
            for (DeployInfoDataDTO dataDTO : grayVersionDataDTOList) {
                if (!"backend".equals(dataDTO.getBackendFrontType())) {
                    continue;
                }
                if (time_loit_servers.contains(dataDTO.getServiceName())) {
                    portalServerList.add(dataDTO);
                } else {
                    businessServerList.add(dataDTO);
                }
            }

            if (CollectionUtils.isNotEmpty(portalServerList)) {
                buildServerConf(portalServerList, "portal");
            }
            if (CollectionUtils.isNotEmpty(businessServerList)) {
                buildServerConf(businessServerList, "business");
            }

        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    public static void buildServerConf(List<DeployInfoDataDTO> list, String folderName) throws IOException {
        String fixedIp = "";
        BackendDataDTO backendDataDTO = new BackendDataDTO();

        for (DeployInfoDataDTO dataDTO : list) {

            Map<String, String> map = new HashMap<>();
            map.put("deployPath", dataDTO.getDeployPath());
            map.put("serviceName", dataDTO.getServiceName());

            if (StringUtils.isBlank(fixedIp)) {
                fixedIp = dataDTO.getFixedIp();
                backendDataDTO = new BackendDataDTO();
                BeanUtils.copyProperties(dataDTO, backendDataDTO);
                backendDataDTO.getMapList().add(map);
                continue;
            }

            if (dataDTO.getFixedIp().equals(fixedIp)) {
                backendDataDTO.getMapList().add(map);
            } else {
                buildBackendConfig(backendDataDTO, folderName);
                buildBashConfig(backendDataDTO, folderName);

                fixedIp = dataDTO.getFixedIp();
                backendDataDTO = new BackendDataDTO();
                BeanUtils.copyProperties(dataDTO, backendDataDTO);
                backendDataDTO.getMapList().add(map);
            }
        }
    }

    /**
     * 生成logstash-es-sone.conf 及其 sh
     */
    private static void buildBackendConfig(BackendDataDTO backendDataDTO, String folderName) throws IOException {

        String serviceName = backendDataDTO.getServiceName();
        String fixedIp = backendDataDTO.getFixedIp();
        List<Map<String, String>> mapList = backendDataDTO.getMapList();

        if (StringUtils.isBlank(serviceName)) {
            return;
        }

        BackendFrontEnum backendFrontType = BackendFrontEnum.getEnumByCode(backendDataDTO.getBackendFrontType());
        if (!BackendFrontEnum.BACKEND.equals(backendFrontType)) {
            return;
        }

        String fileDir = root_path_full + "\\" + fixedIp + "\\elkconfig";
        File folder = new File(fileDir);
        if (!folder.exists()) {
            folder.mkdirs();
        }

        String fileName = "logstash-es-sone-" + folderName + ".conf";

        String logstashFileName = fileDir + "\\" + fileName;

        Map model = new HashMap();
        model.put("mapList", mapList);

        if ("portal".equals(folderName)) {
            model.put("esindex", "portal");
        } else {
            model.put("esindex", "sone");
        }

        String result = FreeMarkerUtils.process("sh\\logstash-conf.ftl", model);
        FileUtils.write(logstashFileName, result);
    }

    /**
     * 生成logstash 启动 停止 脚本
     * @param backendDataDTO
     * @param folderName
     * @throws IOException
     */
    private static void buildBashConfig(BackendDataDTO backendDataDTO, String folderName) throws IOException {

        String serviceName = backendDataDTO.getServiceName();
        String fixedIp = backendDataDTO.getFixedIp();

        if (StringUtils.isBlank(serviceName)) {
            return;
        }

        BackendFrontEnum backendFrontType = BackendFrontEnum.getEnumByCode(backendDataDTO.getBackendFrontType());
        if (!BackendFrontEnum.BACKEND.equals(backendFrontType)) {
            return;
        }

        String fileDir = root_path_full + "\\" + fixedIp;
        File folder = new File(fileDir);
        if (!folder.exists()) {
            folder.mkdirs();
        }

        String fileName = "logstash-es-sone-" + folderName + ".conf";
        String startFileName = "logstash-start-" + folderName + ".sh";
        String stopFileName = "logstash-stop-" + folderName + ".sh";

        String logstashStartFileName = fileDir + "\\" + startFileName;
        String logstashStopFileName = fileDir + "\\" + stopFileName;

        Map model = new HashMap();
        model.put("fileName", fileName);
        model.put("folderName", folderName);

        String startResult = FreeMarkerUtils.process("sh\\logstash-start.ftl", model);
        FileUtils.write(logstashStartFileName, startResult);

        String stopResult = FreeMarkerUtils.process("sh\\logstash-stop.ftl", model);
        FileUtils.write(logstashStopFileName, stopResult);
    }

}
