package com.loit.component.keygen.config;

import com.loit.component.keygen.leaf.LeafSegmentKeyGeneratorFactory;
import com.loit.component.keygen.plugin.AuotIdInterceptor;
import com.sankuai.inf.leaf.IDGen;
import com.sankuai.inf.leaf.common.PropertyFactory;
import com.sankuai.inf.leaf.snowflake.SnowflakeIDGenImpl;
import org.mybatis.spring.boot.autoconfigure.ConfigurationCustomizer;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.Properties;

@Configuration
public class MybatisKeygenConfig {

    /**
     * leaf-segment-loit-test
     */
    @Value("${leaf.namespace}")
    private String leafNamespace;

    /**
     * 127.0.0.1:2181
     */
    @Value("${leaf.zkserver.list}")
    private String serverList;

    @Value("${leaf.segment.id.initial.value}")
    private String leafInitialValue;

    /**
     * 默认2000
     */
    @Value("${leaf.segment.step}")
    private String leafSegmentStep;


    @Value("${server.port}")
    private Integer serverPort;

    @Value("${spring.application.name}")
    private String applicationName;

    @Bean
    public LeafSegmentKeyGeneratorFactory leafSegmentKeyGeneratorFactory() {

        LeafSegmentKeyGeneratorFactory leafSegmentKeyGeneratorFactory = new LeafSegmentKeyGeneratorFactory();

        Properties properties = new Properties();
        properties.setProperty("leaf.namespace", leafNamespace);
        properties.setProperty("server.list", serverList);
        properties.setProperty("leaf.segment.id.initial.value", leafInitialValue);
        properties.setProperty("leaf.segment.step", leafSegmentStep);
        leafSegmentKeyGeneratorFactory.setGlobleProperties(properties);
        return leafSegmentKeyGeneratorFactory;
    }

    @Bean
    public IDGen snowflakeIDGen() {
        Properties properties = PropertyFactory.getProperties();
        properties.setProperty("leaf.zk.list", serverList);
        properties.setProperty("leaf.name", "leaf_snowflake_" + applicationName);

        IDGen idGen = new SnowflakeIDGenImpl(properties.getProperty("leaf.zk.list"), serverPort);
        return idGen;
    }

    /**
     * 插件mybatis id生成 拦截器
     */
    @Bean
    AuotIdInterceptor autoIdInterceptor() {
        return new AuotIdInterceptor();
    }

    @Bean
    ConfigurationCustomizer mybatisConfigurationCustomizer() {
        return configuration -> configuration.addInterceptor(autoIdInterceptor());
    }
}
