
package com.loit.component.keygen.leaf.zk.util;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;
import org.apache.curator.test.TestingServer;
import org.apache.zookeeper.KeeperException;

import java.io.File;
import java.io.IOException;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public final class EmbedTestingServer {
    
    private static final int PORT = 3181;
    
    private static volatile TestingServer testingServer;
    
    /**
     * Start embed zookeeper server.
     */
    public static void start() {
        if (null != testingServer) {
            return;
        }
        try {
            testingServer = new TestingServer(PORT, new File(String.format("target/test_zk_data/%s/", System.nanoTime())));
            // CHECKSTYLE:OFF
        } catch (final Exception ex) {
            // CHECKSTYLE:ON
            if (!isIgnoredException(ex)) {
                throw new RuntimeException(ex);
            }
        } finally {
            Runtime.getRuntime().addShutdownHook(new Thread() {
            
                @Override
                public void run() {
                    try {
                        testingServer.close();
                    } catch (final IOException ignored) {
                    }
                }
            });
        }
    }
    
    private static boolean isIgnoredException(final Throwable cause) {
        return cause instanceof KeeperException.ConnectionLossException || cause instanceof KeeperException.NoNodeException || cause instanceof KeeperException.NodeExistsException;
    }
}
